//
//  RQFauna.java
//  Red Queen Simulator
//
//  Created by John Damuth on Thu Jan 27 2005.
//  Copyright (c) 2005 Pantherion Corporation. All rights reserved.
//  Version 1.5

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

// This initiates a new RQ fauna

public class RQFauna {

	protected static double real_intercept = 4.5;
	protected static double maximum_x_value = 6.0;
	double total_units_of_energy = 50000000.00;
	double sum_of_energy = 0.0;
	double	minimum_viable_energy = 0.0;
	double	minimum_density = 50000000.0;
	double	slope = 0.0;
	double y_int = 0.0;
	double	mean_energy = 0.0;
	double density_residual_variance = 0.0;
	Vector speciesV = new Vector();

	public RQFauna(double tue, int nsp, double yvar, double max, double min, double ks, double bpf, double pfs){
	
		total_units_of_energy = tue;
	
		/* Randomly determine initial slope */
		
	    Random srand = new Random();
		double slope = min + ((max - min) * srand.nextDouble());
		
		for(int i=0;i<nsp;i++){ 

			/* Create a random x-value.  */

            double x = maximum_x_value * srand.nextDouble();
                
            /* Create the y-value based on the model.  */

            double y = (slope * x) + real_intercept;

            /* Adjust the y-value according to y_residual_variance. */

				/* Gaussian distribution based on sum 12 random #s  */
                /*  (Miller, 1981 Basic Programs for Scientists and */
                /*  Engineers, Sybex, Berkeley, Chapter 2).         */

			double sumrands = 0.0;
			for(int j=0;j<12;j++){
				sumrands = sumrands + srand.nextDouble();
				}
			double gaussian_rand = (sumrands - 6.0) * (Math.sqrt(yvar));
			y = y + gaussian_rand;

			// Create one of the fauna's species
			
			RQSpecies sp = new RQSpecies(Math.pow(10.0,y),Math.pow(10.0,x), ks, bpf, pfs);
			speciesV.addElement(sp);
			sum_of_energy = sum_of_energy + sp.getEnergy();
             }
			
		/*  Rescale energy so that total is the same for each run. */
		/* That is, each fauna gets the same total amount of       */
		/* energy to play with.                                    */
		
		double minimum_ind_energy = total_units_of_energy; 
			
		for(int i=0;i<nsp;i++){ 
			RQSpecies sp = (RQSpecies)speciesV.elementAt(i);
			sp.setEnergy((total_units_of_energy/sum_of_energy) * sp.getEnergy());
			if(sp.getEnergy() < minimum_ind_energy) minimum_ind_energy = sp.getEnergy();
			if(sp.getDensity() < minimum_density) minimum_density = sp.getDensity();			
			}
			
		/* Scale minimum viable energy to the observed minimum */
	
		minimum_viable_energy = 0.5 * minimum_ind_energy;

		slope = calculate_slope();
		}

	public double calculate_mean_energy(){
	
		double	sum_e = 0.0;
		int		np=0;
		
        for(int i=0;i<speciesV.size();i++){
        
        	RQSpecies sp = (RQSpecies)speciesV.elementAt(i);
        	if(!sp.getExtinct()){
        		np++;
        		sum_e = sum_e + sp.getEnergy();
                }
			}	
		return (double)(sum_e/np);
		}

	public double calculate_slope(){
	
		double          var_x;
		double          var_y;
		double          mean_x;
		double          mean_y;
		int				np=0;
		double          sumxy = 0.0;
		double          sumx2 = 0.0;
		double          sumy2 = 0.0;
		double          sumx = 0.0;
		double          sumy = 0.0;
		double          covxy = 0.0;
		double			byx=0.0;
		double			bxy=0.0;
		double			x, y;
		double			r, r2;
 
        for(int i=0;i<speciesV.size();i++){
        
        	RQSpecies sp = (RQSpecies)speciesV.elementAt(i);
        	if(!sp.getExtinct()){
        		np++;
        		y = log10(sp.getDensity());
				x = log10(sp.getMass());        
                sumx = sumx + x;
                sumy = sumy + y;
                sumx2 = sumx2 + (x * x);
                sumy2 = sumy2 + (y * y);
                sumxy = sumxy + (x * y);              
                }
			}

        mean_x = sumx / np;
        mean_y = sumy / np;

        var_x = (sumx2 - ((sumx * sumx) / np)) / (np - 1);
        var_y = (sumy2 - ((sumy * sumy) / np)) / (np - 1);

		byx = (sumxy - ((sumx * sumy) / np)) / (sumx2 - ((sumx * sumx)/ np));
        bxy = (sumxy - ((sumx * sumy) / np)) / (sumy2 - ((sumy * sumy)/ np));
        r = Math.sqrt((byx * bxy));
        r2 = r * r;
        covxy = byx * var_x;
        
        y_int = mean_y - (byx * mean_x);
        double mse =  ((np - 1) / (np - 2)) * (var_y - ((byx * byx) * var_x));
        double se = Math.sqrt(mse) / ((Math.sqrt(var_x)) * Math.sqrt(np - 1));
		
		density_residual_variance = mse;
        
		return byx;	
		}

	public double log10(double x) {
  		return Math.log(x)/Math.log(10.0);  
		}

	public double getSlope(){
		return slope;
		}
		
	public double getYint(){
		return y_int;
		}

	public double getMinDensity(){
		return minimum_density;
		}
		
	public Vector getSpecies(){
		return speciesV;
		}
		
	public double getMinimumViableEnergy(){
		return minimum_viable_energy;
		}

	public double getMeanEnergy(){
		return mean_energy;
		}
		
	public double getYresidVar(){
		return density_residual_variance;
		}

}