//
//  RQSpecies.java
//  Red Queen Simulator
//
//  Created by John Damuth on Thu Jan 27 2005.
//  Copyright (c) 2005 Pantherion Corporation. All rights reserved.
//  Version 1.5

import java.util.*;

// This initiates a new RQ species

public class RQSpecies {

	double density, mass, energy;
	double acquisition_factor = 1.0;
	double protection_factor = 1.0;
	double kleiber_slope;
	boolean	extinct = false;
	boolean	lost_interaction = false;
	boolean isnew = false;
	
	double base_protection_factor = 1.0;
	double pf_slope;

	public RQSpecies(double den, double mas, double ks, double bpf, double pfs){
	
		density = den;
		mass = mas;
		kleiber_slope = ks;
		base_protection_factor = bpf;
		pf_slope = pfs;
		energy = convert_density_to_energy(density, mass);
		protection_factor = calculate_protection_factor(energy);
		}
		
	public void setDensity(double den){
	
		density = den;
		energy = convert_density_to_energy(density, mass);
		protection_factor = calculate_protection_factor(energy);
		}
		
	public void setEnergy(double enrg){	
		energy = enrg;
		density = convert_energy_to_density(energy, mass);
		protection_factor = calculate_protection_factor(energy);
		}
		
	public void setAcquisitionFactor(double acqf){
		acquisition_factor = acqf;
		}
		
	public void setExtinct(boolean ext){
		extinct = ext;
		}
		
	public void setLost(boolean lst){
		lost_interaction = lst;
		}
			
	public void setNew(boolean nw){
		isnew = nw;
		}
			
	public void setMass(double ms){
		mass = ms;
		}			

	public double getEnergy(){
		return energy;
		}
		
	public double getDensity(){
		return density;
		}
		
	public double getMass(){
		return mass;
		}

	public double getAcquisitionFactor(){
		return acquisition_factor;
		}		

	public boolean getExtinct(){
		return extinct;
		}
		
	public boolean getLost(){
		return lost_interaction;
		}
	
	public double getProtectionFactor(){
		return protection_factor;
		}
		
	public boolean IsNew(){
		return isnew;
		}	
	
	double convert_density_to_energy(double density, double mass){

		double		nrg;  /*  MJ/yr  */

		mass = mass/1000.0;

		/*  Uses simple Kleiber scaling (Kleiber, 1975,   */
		/* 'The Fire of Life'), which is (4.1) x M^0.75   */
		/*  J/sec. =  130 MJ/yr                           */

		nrg = (4.1 * (Math.pow(mass,kleiber_slope))) * density * 31.56;
		
		if(nrg <= 0){
//			System.out.println("    Energy was calculated <= 0 , to wit " + nrg);
			}

		return nrg;
		}
	
	
	double convert_energy_to_density(double energy, double mass){

		double		dsty;  /*  MJ/yr  */

		mass = mass/1000.0;

		/*  Uses simple Kleiber scaling (Kleiber, 1975,   */
		/* 'The Fire of Life'), which is (4.1) x M^0.75   */
		/*  J/sec. =  130 MJ/yr                           */

		dsty = (energy/31.56) / (4.1 * (Math.pow(mass,kleiber_slope)));

		if(dsty <= 0){
//			System.out.println("    Density was calculated <= 0, to wit " + dsty);
			}
		
		return dsty;
		}

	double calculate_protection_factor(double energy){

		double	new_p_factor;
		
		/* THIS IS NOT USED --- LEFT OVER FROM EARLY DEVELOPMENT  */
		
		/*   Calculate the "protection-factor" for each species, based on its  */
		/* energy and the user-specified base_protection_factor [value of 1 by */
		/* default] and the pf_slope [default value 0].  If the pf_slope is 0, */
		/* all species get the same (base) protection factor.  This can be     */
		/* adjusted by [pf_slope * energy] in order to mimic a jack-of-all     */
		/* trades limitiation as a linear function of current energy-use. That */
		/* is, it represents a factor affecting the probability that a species */
		/* will lose in an interaction.                                        */
		/*   The new protection factor is calculated in logarithmic space,     */
		/* according to the slope specified by the user. If the slope is 0, no */
		/* change in the protection factor takes place.                        */
	
		new_p_factor = log10(base_protection_factor) - (pf_slope * log10(energy)); 

		return Math.pow(10, new_p_factor);

		}


	public double log10(double x) {
  		return Math.log(x)/Math.log(10.0);  
		}

	
}
