//
//  RQrun.java
//  Red Queen Simulator
//
//  Created by John Damuth on Thu Jan 27 2005.
//  Copyright (c) 2005-2007 Pantherion Corporation. All rights reserved.
//  Version 1.5

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.io.*;
import javax.imageio.ImageIO;
import java.text.*;

import java.util.*;


// This initiates a new RQ run and executes it

public class RQrun {

	MainPanel	parent;
	int		number_of_faunas;
	int		timesteps;
	double	energy_acquisition_increment;
	double	energy_acquisition_multiple;
	double	energy_acquisition_constant;
	double	minimum_viable_energy;
	double	extinction_energy_threshold;
	double	protection_energy_threshold;
	double	acquisition_energy_threshold;
	double	max_initial_slope;
	double	min_initial_slope;
	double	base_acquisition_factor;
	double	base_protection_factor;
	double	base_extinction_prob;
	double	pf_slope;
	double	extinction_density_limit;
	double	minimum_density;
	double  y_int;
	double	seeds[] = new double[10001];
	
	/*  Program States */

	double	pool_of_unused_energy;
	int		number_of_species;
	boolean	RQ_interactions;
	boolean	proportional_ex;
	boolean	random_ex;
	int		number_extinct;
	int		extinct_species;
	boolean	universe_open;
	boolean	runs_mode;
	boolean	use_min_density;
	boolean	poolvar_is_constant;

	double	competition_range;
	double	resurrection_slope;
	double	aeThresh_pct;
	double	num_advancing;
	int		num_interactions;
	int		e_grab_tot;
	int		sd_index = 0;
	RQFauna fauna;
	
// Curve fitting variables


	double          real_slope;
	double          real_intercept;
	double          x_residual_variance;
	double          y_residual_variance;
	double          se;
	double			maximum_x_value;
	double			mse;


	protected static double total_units_of_energy = 50000000.00;

/*******************************/
/*         Constructor         */
/*******************************/	


	public RQrun(MainPanel prnt){
	
		parent = prnt;
		
		number_of_faunas = prnt.FaunasSlider.getValue();
		number_of_species = prnt.SpeciesSlider.getValue();
		timesteps = prnt.TimestepsSlider.getValue();
		y_residual_variance = prnt.YvarianceSlider.getValue() *0.01;
		max_initial_slope = prnt.MaxSlopeSlider.getValue() * 0.01;
		min_initial_slope = prnt.MinSlopeSlider.getValue() * 0.01;
		base_protection_factor = (prnt.BasePFSlider.getValue() * 0.1) + 1.0;
		pf_slope = prnt.PFSlopeSlider.getValue() * 0.01;  // Not used
		base_acquisition_factor = 1.0;
		acquisition_energy_threshold = prnt.AdvancingSlider.getValue();
		energy_acquisition_multiple = prnt.AcquisitionSlider.getValue() * 0.1; //Not used
		energy_acquisition_increment = prnt.EnergySlider.getValue() * 0.1;
		double kleiber_slope = prnt.KleiberSlider.getValue() * 0.01;
		proportional_ex = prnt.exchange_proportional;  // Not used
		random_ex = prnt.evalue_random;
		universe_open = prnt.universe_open;
		use_min_density = prnt.minimum_is_density;
		extinction_density_limit = prnt.ExtinctionLimitSlider.getValue() * 1.0;
		competition_range = prnt.CompetitionSlider.getValue() * 0.025;  // Not used
		poolvar_is_constant = prnt.poolvar_is_constant;
		
		
		energy_acquisition_constant = (energy_acquisition_increment * total_units_of_energy / number_of_species) - (total_units_of_energy / number_of_species);

		num_interactions = ((number_of_species * number_of_species) - number_of_species);		
		
		total_units_of_energy = 5*Math.pow(10,prnt.EtotSlider.getValue());
//			System.out.println("Total Energy = " + total_units_of_energy);		
		load_seeds();
		
		double sumx[] = new double[10001];
		double sumx2[] = new double[10001];
		double summ[] = new double[10001];
		double summ2[] = new double[10001];
		double sumeu[] = new double[10001];
		double sumeu2[] = new double[10001];
		double sumN[] = new double[10001];
		
		// Create each fauna of the run
		//  and have it evolve
		
		
		
		for(int i=0;i< number_of_faunas;i++){
		
		// If the max and min slopes are specified as equal, makes actual starting slopes very close
		//  ( within .001) to the specified value. Otherwise, they range farther, randomly
		
			for(;;){
			fauna = new RQFauna(total_units_of_energy, number_of_species,y_residual_variance,max_initial_slope,min_initial_slope, kleiber_slope, base_protection_factor, pf_slope);
			if(max_initial_slope == min_initial_slope){
					double limitedSlope = fauna.calculate_slope();
				if(limitedSlope > min_initial_slope-.001 && limitedSlope < max_initial_slope+.001) break;
				}
			else{
				break;
				}
			
			
			
			}
			
			resurrection_slope = fauna.calculate_slope();
			
			sumx[0] = sumx[0] + resurrection_slope;
			sumx2[0] = sumx2[0] + (resurrection_slope * resurrection_slope);
			
			// Set the initial states for the Stdev and Seud cases to 0 for timestep 0;
			//  This is a kluge and just results in no data for the first stimestep
			summ[0] = 0.0;  
			summ2[0] = 0.0; 
			sumeu[0] = 0.0;
			sumeu2[0] = 0.0;
			sumN[0] = 0.0;
			
			minimum_viable_energy = fauna.minimum_viable_energy;
			maximum_x_value = fauna.maximum_x_value;
			
			prnt.CRT.slcrt.setTimesteps(timesteps);
			prnt.CRT.sdcrt.setTimesteps(timesteps);
			
			for(int j=1;j<timesteps+1;j++){
				if(!prnt.should_execute) break;
				if(!universe_open) evolve(j, i, fauna);
				else random_evolve(j, i, fauna, prnt.hard_cap_open);
				double slope = fauna.calculate_slope();
				resurrection_slope = slope;
				if(prnt.slope_view){
					if(prnt.analysis_on){    // The Standard Deviation CRT
						prnt.CRT.sdcrt.draw_sdev(j, slope, fauna.getYint(), fauna.getSpecies(), fauna.getMinDensity());
						sumx[j] = sumx[j] + prnt.CRT.sdcrt.getSdev();
						sumx2[j] = sumx2[j] + (prnt.CRT.sdcrt.getSdev() * prnt.CRT.sdcrt.getSdev());
						summ[j] = summ[j] + prnt.CRT.sdcrt.getMean();
						summ2[j] = summ2[j] + (prnt.CRT.sdcrt.getMean() * prnt.CRT.sdcrt.getMean());
						sumeu[j] = sumeu[j] + prnt.CRT.sdcrt.getEuler();
						sumeu2[j] = sumeu2[j] + (prnt.CRT.sdcrt.getEuler() * prnt.CRT.sdcrt.getEuler());
						sumN[j] = sumN[j] + prnt.CRT.sdcrt.getN();
						}
					else{
						prnt.CRT.slcrt.draw_slope(j, slope);  //Ordinary slope CRT
						sumx[j] = sumx[j] + slope;
						sumx2[j] = sumx2[j] + (slope * slope);
						}
					}
				else{
					if(prnt.analysis_on){  //SEUD CRT
						prnt.CRT.seudcrt.draw_graph(j, slope, fauna.getYint(), fauna.getSpecies(), fauna.getMinDensity());
						sumx[j] = sumx[j] + prnt.CRT.seudcrt.getSdev();
						sumx2[j] = sumx2[j] + (prnt.CRT.seudcrt.getSdev() * prnt.CRT.seudcrt.getSdev());
						summ[j] = summ[j] + prnt.CRT.seudcrt.getMean();
						summ2[j] = summ2[j] + (prnt.CRT.seudcrt.getMean() * prnt.CRT.seudcrt.getMean());
						sumeu[j] = sumeu[j] + prnt.CRT.seudcrt.getEuler();
						sumeu2[j] = sumeu2[j] + (prnt.CRT.seudcrt.getEuler() * prnt.CRT.seudcrt.getEuler());
						sumN[j] = sumN[j] + prnt.CRT.seudcrt.getN();
						}
					else{   // Ordinary Points CRT
						prnt.CRT.pcrt.draw_graph(j, slope, fauna.getYint(), fauna.getSpecies(), fauna.getMinDensity());
						sumx[j] = sumx[j] + slope;
						sumx2[j] = sumx2[j] + (slope * slope);
						}
					}			

			}
		
	
		}

		String wd = System.getProperty("user.dir");
		prnt.SaveButton.setIcon(prnt.saveB);
		prnt.SaveButton.setEnabled(true);
		String fs = get_file_sequence(wd);
		String output_file = new String(wd + File.separator + "rq" + fs + "output.txt");
		String params_file = new String(wd + File.separator + "rq" + fs + "params.txt");
		
		if(number_of_faunas > 2){		
		try{
			FileOutputStream os = new FileOutputStream(output_file, false);
			PrintWriter out = new PrintWriter(os);
			if(prnt.CRT.slcrt.isVisible() || prnt.CRT.pcrt.isVisible()){
				out.println("Timestep\tMean Slope\t95%CI");
				for(int k=0;k<timesteps;k++){
					double mean = sumx[k]/number_of_faunas;
					double ci95 = 1.96 * Math.sqrt((sumx2[k] - ((sumx[k] * sumx[k])/number_of_faunas))/(number_of_faunas - 1));
					out.println(k + "\t" + mean + "\t" + ci95);
					}
				}
			else{
				out.println("Timestep\tMean SDEV\t95%CI sd\tMean Log10 Mean e-use\t95%CI mlm\tMean Euler\t95%CI euler\tMean S");
				for(int k=0;k<timesteps;k++){
					double mean_sd = sumx[k]/number_of_faunas;
					double ci95sd = 1.96 * Math.sqrt((sumx2[k] - ((sumx[k] * sumx[k])/number_of_faunas))/(number_of_faunas - 1));
					double mean_m = summ[k]/number_of_faunas;
					double ci95m = 1.96 * Math.sqrt((summ2[k] - ((summ[k] * summ[k])/number_of_faunas))/(number_of_faunas - 1));
					double mean_Eu = sumeu[k]/number_of_faunas;
					double ci95eu = 1.96 * Math.sqrt((sumeu2[k] - ((sumeu[k] * sumeu[k])/number_of_faunas))/(number_of_faunas - 1));
					double mean_N = sumN[k]/number_of_faunas;
					out.println(k + "\t" + mean_sd + "\t" + ci95sd + "\t" + mean_m + "\t" + ci95m + "\t" + mean_Eu + "\t" + ci95eu + "\t" + mean_N);
					}
				}
			out.close();
			os.close();
			}
		catch(IOException e){
			System.out.println("Error writing confidence intervals file");
			}
		}
		
		try{
			FileOutputStream os = new FileOutputStream(params_file, false);
			PrintWriter out = new PrintWriter(os);
			StringBuffer sb = new StringBuffer();
			
			DecimalFormat df = new DecimalFormat("###.##");
			String maxSlope = df.format(max_initial_slope);
			String minSlope = df.format(min_initial_slope);
			
			String ls = System.getProperty("line.separator");
			
			sb.append("number_of_faunas = " + number_of_faunas + ls);
			sb.append("number_of_species = " + number_of_species + ls);
			sb.append("timesteps = " + timesteps + ls);
			sb.append("y_residual_variance = " + y_residual_variance + ls);
			sb.append("max_initial_slope = " + maxSlope + ls);
			sb.append("min_initial_slope = " + minSlope + ls);
			sb.append("total_units_of_energy = " + total_units_of_energy + ls);
			sb.append("acquisition_energy_threshold = " + acquisition_energy_threshold + ls);
			sb.append("energy_acquisition_increment = " + energy_acquisition_increment + ls);
			sb.append("kleiber_slope = " + kleiber_slope + ls);
			sb.append("random_ex = " + random_ex + ls);
			sb.append("no interaction = " + universe_open + ls);
			sb.append("energy cap = " + prnt.hard_cap_open + ls);
			sb.append("use_min_density = " + use_min_density + ls);
			sb.append("extinction_density_limit = " + extinction_density_limit + ls);
			sb.append("pool variance is constant = " + poolvar_is_constant + ls);

			out.println(sb.toString());
		
			out.close();
			os.close();
			}
		catch(IOException e){
			System.out.println("Error writing parameters file");
			}
				
		
		
		prnt.RunningLight.setIcon(prnt.light_off);
	}
	
/*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^*/
String get_file_sequence(String wd){

	/**************************************/
	/*  This simply checks the output     */
	/* directory for previous runs and    */
	/* increments the file names to be    */
	/* unique and in sequence.            */
	/**************************************/

	String [] current_files;
	int sequence_index = 0;
	String index;
	
	File dir = new File(wd);

	FilenameFilter filter = new FilenameFilter() {
        public boolean accept(File dir, String name) {
            return name.endsWith("output.txt");
        }
    };
	
	current_files = dir.list(filter);
	
	if (current_files == null){
		}
	else{
		sequence_index = current_files.length + 1;
		}
		
	if(sequence_index < 10){
		index = new String("00" + sequence_index);
		}
	else if (sequence_index > 9 && sequence_index < 100){
		index = new String("0" + sequence_index);
		}
	else{
		index = new String("" + sequence_index);
		}
	
return index;


}
/*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^*/
	void load_seeds() {

		Random srand = new Random();

		for(int i=0;i<10000;i++){
			seeds[i] = srand.nextDouble();
			}
		
		sd_index = 0;
	}
/*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^*/
	double get_seed() {
		if(sd_index >= 9999) load_seeds();
	
		sd_index++;
	
		return seeds[sd_index];
	}
/*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^*/
	public void quick_sort(RQArrayPoint array[], int first, int last){
	
		int low = first;
		int high = last;
		
		if (first > last) return;
		
		int mid = array[(first + last)/2].rnumber;
		
		do {
			while (array[low].rnumber < mid) low++;
			
			while (array[high].rnumber > mid) high--;
			
			if (low <= high){
				RQArrayPoint temp = array[low];
				array[low++] = array[high];
				array[high--] = temp;
				}
				
			} while (low <= high);
			
		quick_sort(array, first, high);
		quick_sort(array, low, last);
				
	
	
	}

/*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^*/
	void evolve(int timestep, int fauna_number, RQFauna fauna){
	
		/*  Randomize array order */
		
		RQArrayPoint actors[] = new RQArrayPoint[number_of_species];
		RQArrayPoint victims[] = new RQArrayPoint[number_of_species];
		
		for (int i=0; i< number_of_species; i++){
			actors[i] = new RQArrayPoint(i, (int)(Math.random() * 1000));
			victims[i] = new RQArrayPoint(i, (int)(Math.random() * 1000));
//			System.out.println(array[i].index + "  " + array[i].rnumber);
			}
			
		quick_sort(actors, 0, actors.length-1);	
		quick_sort(victims, 0, victims.length-1);	


		Vector sp = fauna.getSpecies();

		/********************/
		/*   Round Begins   */
		/********************/

		for(int i=0;i<number_of_species;i++){
		
			RQSpecies actor = (RQSpecies)sp.elementAt(actors[i].index);
			actor.setNew(false);
		
			for(int j=0;j<number_of_species;j++){
			
				/* Each actor is checked against each potential  */
				/*  victim in sequence.                          */
				/* Don't let a species act upon itself, or try   */
				/*  to extract energy from an extinct species.   */
						
				if (actors[i].index != victims[j].index){
				
					RQSpecies victim = (RQSpecies)sp.elementAt(victims[j].index);
										
					if (!victim.getExtinct()){				
						check_interaction(actor, victim, timestep);
						}
					}
				}
		
			}

	
	
	}
/*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^*/	
	void random_evolve(int timestep, int fauna_number, RQFauna fauna, boolean hardcap_open){

	/***********************************************************/
	/*  This simulates an "open" universe, where species gains */
	/* and losses in energy control are independent of each    */
	/* other. Each species is playing its own game, random     */
	/* walking to great abundance or to extinction. The major  */
	/* reason to include this is to demonstrate that there is  */
	/* in how the simulation is set up that would dictate the  */
	/* results, other than evolutionary Red Queen interactions */
	/* among species. This section is also used to process the */
	/* "No Interaction" runs in a constant energy situation.   */
	/***********************************************************/

	/* We start in the usual way */
double num_advancing = (acquisition_energy_threshold/100.0) * number_of_species;
double criterion = num_advancing/num_interactions;
Random gr = new Random();
double sign = -1.0;


		/*  Randomize array order */
		
		RQArrayPoint actors[] = new RQArrayPoint[number_of_species];
		RQArrayPoint victims[] = new RQArrayPoint[number_of_species];
		
		for (int i=0; i< number_of_species; i++){
			actors[i] = new RQArrayPoint(i, (int)(Math.random() * 1000));
			victims[i] = new RQArrayPoint(i, (int)(Math.random() * 1000));
			}
			
		quick_sort(actors, 0, actors.length-1);	
		quick_sort(victims, 0, victims.length-1);	

		
		Vector sp = fauna.getSpecies();

		/***************************/
		/*   Random Round Begins   */
		/***************************/

		for(int i=0;i<number_of_species;i++){
		
			RQSpecies actor = (RQSpecies)sp.elementAt(actors[i].index);
			for(int j=0;j<number_of_species;j++){
			
				/* Each actor is checked against each potential  */
				/*  victim in sequence.                          */
				/* Don't let a species act upon itself, or       */
				/*  be tested against an extinct species.        */
				
				/* Note: This process is gone through here only  */
				/* determine the "winners"                       */
						
				if (actors[i].index != victims[j].index){
				
					RQSpecies victim = (RQSpecies)sp.elementAt(victims[j].index);
										
					if (!victim.getExtinct()){				
						/* Instead of the full check-interaction function */
						double die_roll = Math.random();						

						if (die_roll <= criterion * actor.getAcquisitionFactor() * (1.0/victim.getProtectionFactor()) ){
							/* Actor wins --- its energy will be changed */
							if(actor.getExtinct()){  // In this case create a "new" species and just give it the energy it comes with
								create_new_species(actor,fauna);
								}
							else{
								/* Increase or decrease the species' energy by the specified energy increment */
								/* If "hard_cap_open" is not chosen, total amount of energy can change without limit */
								if (gr.nextDouble() <= 0.50) sign = -1.0;
								else sign = 1.0;
			
								double energy_increment = random_energy(energy_acquisition_increment);
								energy_increment = Math.abs(energy_increment);

								double new_energy = log10(actor.getEnergy()) + (energy_increment/100 * sign);
								actor.setEnergy(Math.pow(10,new_energy));
								if((!use_min_density && actor.getEnergy() <= minimum_viable_energy) ||
									(use_min_density && actor.getDensity() <= extinction_density_limit)){
									actor.setExtinct(true);
									}
									
								// If chosen, hard_cap_open makes sure that the system as a whole is closed --- the total
								//   amount of energy remains effectively constant, in the same way that each
								//   fauna was scaled to a constant energy level when it was created. In this case,
								//   when a winning species changes energy, ALL species' energy is equally adjusted
								//   to keep the total in line. So, one can see the effect of closure of the system
								//   as a whole	independently of the winner-loser system in the ordinary closed mode.
								if(hardcap_open){
									double sm_of_energy = 0.0;
									for(int g=0;g<number_of_species;g++){
										RQSpecies spec = (RQSpecies)sp.elementAt(g);
										if(!spec.getExtinct()){
										sm_of_energy = sm_of_energy + spec.getEnergy();
											}
										}
									for(int g=0;g<number_of_species;g++){
										RQSpecies spec = (RQSpecies)sp.elementAt(g);
										if(!spec.getExtinct()){
											spec.setEnergy((total_units_of_energy/sm_of_energy) * spec.getEnergy());
										
											}
										}
//System.out.println("SOE = " + sm_of_energy);										

									}
								}
							}
						}
					}
				}
			}	
	
	}
/*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^*/	
	void check_interaction(RQSpecies actr, RQSpecies victm, int timestep){
	
	RQSpecies actor, victim;
	
	double die_roll = Math.random();
	double num_advancing = (acquisition_energy_threshold/100.0) * number_of_species;
	double criterion = num_advancing/num_interactions;
	double energy_grab = 0.0;
	double log_new_energy = 0.0;
	double energy_increment = 0.0;

	actor = actr;
	victim = victm;

	/* Determine whether species "actor" takes energy from */
	/*  species "victim".  If, so, perform any necessary   */
	/*  parameter adjustments.                             */
	
	/* The "die_roll" is a randomly chosen number    */
	/*   within the range of the total number of     */
	/*   tested interactions (permutations) of the   */
	/*   species.  That is, with 120 species there   */
	/*   will be 14280 interactions, a specified %   */
	/*   of which in each time interval are expected */
	/*   to be successful (the num_advancing).  If   */
	/*   the die_roll is <= to this number, the      */
	/*   interaction is a success. In older versions */
	/*   the num_advancing would be affected by the  */
	/*   acquisition and protection factors, but now */
	/*   these are not used.                         */

	
	// Determine whether actor wins interaction
	
	if (die_roll <= criterion * actor.getAcquisitionFactor() * (1.0/victim.getProtectionFactor()) ){
	
		// Actor wins, determine the size of the energy grab

		if(actor.getExtinct()){  // In this case create a "new" species
			boolean not_close_enough = true;
			if(competition_range == 0){
				create_new_species(actor, fauna);
				}
			else{
				while(not_close_enough){              // Keep making new species until one is close enough in body size
					create_new_species(actor,fauna);
					if(Math.abs(log10(actor.getMass()) - log10(victim.getMass())) - competition_range <= 0){
						not_close_enough = false;
						}
					}
				}
			energy_grab = actor.getEnergy();
			actor.setEnergy(0.0);  // The actor will be given his energy (the energy_grab) back further on
			}
		else if(random_ex && proportional_ex){
			actor.setNew(false);
			energy_increment = random_energy(energy_acquisition_increment);
			energy_increment = Math.abs(energy_increment);
			log_new_energy = log10(actor.getEnergy()) + log10((energy_increment/100.0) + 1.0);
			energy_grab = Math.pow(10, log_new_energy) - actor.getEnergy();	
			}
		else if(!random_ex && !proportional_ex){
		actor.setNew(false);
			energy_grab = energy_acquisition_constant;
			log_new_energy = log10(actor.getEnergy() + energy_grab);
			}
		else if(random_ex && !proportional_ex){
		actor.setNew(false);
			energy_increment = Math.abs(random_energy(energy_acquisition_constant)); // Constant because it must be scaled to the average energy of species
			energy_grab = energy_increment;
			log_new_energy = log10(actor.getEnergy() + energy_grab);
			}
		else if(!random_ex && proportional_ex){ 
		actor.setNew(false);
			log_new_energy = log10(actor.getEnergy()) + log10((energy_acquisition_increment/100.0) + 1.0);
			energy_grab = Math.pow(10, log_new_energy) - actor.getEnergy();
			}
		
		if(competition_range > 0 && Math.abs(log10(actor.getMass()) - log10(victim.getMass())) - competition_range > 0) return;	
			
		// Transfer energy from victim to actor --- if victim loses too much it goes extinct
		//   in which case the actor gets all the loser's energy, but no more (even if energy-grab
		//   was originally larger).
		
		double losers_original_energy = victim.getEnergy();
		double actors_original_energy = actor.getEnergy();
			
		victim.setEnergy(losers_original_energy - energy_grab);
		if((!use_min_density && victim.getEnergy() <= minimum_viable_energy) ||
			(use_min_density && victim.getDensity() <= extinction_density_limit)){
			victim.setExtinct(true);
			victim.setNew(false);
			double new_energy = actors_original_energy + losers_original_energy;
			actor.setEnergy(new_energy);			
			}
		else{
			double new_energy = actors_original_energy + energy_grab;
			actor.setEnergy(new_energy);
			}

		}
	
	}
/*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^*/
void create_new_species(RQSpecies actor, RQFauna fauna){


			actor.setNew(true);			
			actor.setMass(Math.pow(10,(Math.random() * maximum_x_value))); 	// randomly chosen mass		
						
			/**************************************************************/
			/*  In versions 1.0-1.1 the following code gave a new species */
			/* an average amount of energy, regardless of body size. This */
			/* could help drive the slope towards the EER as new species  */
			/* enter drawn from an EER universe. Version 1.2+ draws new   */
			/* species energy from a universe based on the current faunal */
			/* slope, so that, on average, new species will not have any  */
			/* net effect on the density slope.                           */
			/**************************************************************/
			
			
			// The new species has a density predicted from the current regression slope and intercept,
			// plus a random deviation depending on y-variance [NEW in 1.3!], which is then translated into an energy grab.
			// Since the new species is at the moment not using any energy locally, its starting value is set to 0
			
			double predicted_log_density = (log10(actor.getMass()) * fauna.getSlope()) + fauna.getYint();
			
			// If the y-variance is set to be constant, use the original y-variance of the run,
			//   but if "variable" use the current y-residual-variance of the fauna.
			
			double y_variance_used;
			
			if(poolvar_is_constant){
				y_variance_used = y_residual_variance;
				}
			else{
				y_variance_used = fauna.getYresidVar();
				}
			
			Random srand = new Random();
			double sumrands = 0.0;
			for(int j=0;j<12;j++){
				sumrands = sumrands + srand.nextDouble();
				}
			double gaussian_rand = (sumrands - 6.0) * (Math.sqrt(y_variance_used));		
			double log_density = predicted_log_density + gaussian_rand;
			actor.setDensity(Math.pow(10,log_density));			
			
			
			
						
			actor.setExtinct(false);

}

/*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^*/	
	double random_energy(double base_energy){
	
		double sign = 0.0;
		
		Random gr = new Random();
		
        double gaussian_rand = gr.nextGaussian() * (Math.sqrt(base_energy/10.0));
        base_energy = base_energy + gaussian_rand;

		if(universe_open){
			if(gr.nextDouble() <= 0.50) sign = -1.0;
			else sign = 1.0;
			base_energy = base_energy * sign; 
			}		

        if(base_energy == 0) base_energy = 0.00000001; 

return base_energy;
}

/*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^*/
	public double log10(double x) {
  		return Math.log(x)/Math.log(10.0);  
		}
/*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^*/
	public void saveVideoData(File sd){

	/****************************************************/
	/*  The "Slopes" screen is saved as a PNG and the   */
	/* "Points" screen is saved as the data for each    */
	/* species on the final screen, in a tab-delimited  */
	/* text file.                                       */
	/****************************************************/

		File savedata = sd;
			
		if(parent.VideoModeSwitch.getIcon() == parent.switch_off){

			Point loc = parent.CRT.getLocationOnScreen();

			try{
			
				Robot xx = new Robot();
				Rectangle ss = new Rectangle(loc.x,loc.y,parent.CRT.getWidth(),parent.CRT.getHeight());
				javax.imageio.ImageIO.write(xx.createScreenCapture(ss), "png", savedata);
				}
			catch (Exception e) {
				System.err.println(e);
				}				
			}
		else {
			Writer output = null;
			try {
				output = new BufferedWriter( new FileWriter(savedata) );
				if(parent.CRT.pcrt.isVisible()){
					output.write("Species\tMass\tDensity\n");
					for(int i=0;i<fauna.speciesV.size();i++){
						RQSpecies sp = (RQSpecies)fauna.speciesV.elementAt(i);
						if(!sp.getExtinct()){
							output.write("" + (i+1) + "\t" + sp.getMass() + "\t" + sp.getDensity() + "\n");
							}
						}
					}
				else if(parent.CRT.seudcrt.isVisible()){
					output.write("Species\tMass\tEnergy-Use\n");
					for(int i=0;i<fauna.speciesV.size();i++){
						RQSpecies sp = (RQSpecies)fauna.speciesV.elementAt(i);
						if(!sp.getExtinct()){
							output.write("" + (i+1) + "\t" + sp.getMass() + "\t" + sp.getEnergy() + "\n");
							}
						}
					}
				
				}
			catch (IOException ex){
				ex.printStackTrace();
				}
			finally {
				try{
					if (output != null) output.close();
					}			
			catch (IOException ex){
					ex.printStackTrace();
					}
				}			       

			}
		
		
		
		
		}
	
}