//
//  SdevCRT.java
//  Red Queen Simulator
//
//  Created by John Damuth on Thu Jan 27 2005.
//  Copyright (c) 2005 Pantherion Corporation. All rights reserved.
//  Version 1.5

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.io.*;
import java.util.*;
import java.text.DecimalFormat;

public class SdevCRT extends Canvas {

	Dimension	size;
	Color 		drawColor = new java.awt.Color(175, 210, 40);
	int			timesteps;
	static int lastx;
	static int lasty;
	double mean_arithmetic = 0.0;
	double mean_of_dist = 0.0;
	double stdev_of_dist = 0.0;
	double euler_diff = 0.0;
	int bigN = 0;

	public SdevCRT(Dimension sz){
	
		size = sz;
		setBackground(new java.awt.Color(30, 125, 30));
		setSize(size);
		setFont(new java.awt.Font("Helvetica", 0, 10));
		setLocation(new java.awt.Point(5, 8));
		}


	public void setTimesteps(int ts){
		timesteps = ts;
		if(this.isVisible()) repaint();
		}

	public void ClearDisplay(){
		Graphics g = this.getGraphics();
		
		g.clearRect(0,0,size.width,size.height);	
		paint(g);
		}

	public void draw_sdev(int timestep, double slope, double yint, Vector sl, double min_den){	
		Graphics g = this.getGraphics();
		g.setColor(drawColor);
		g.translate(40,20);
		
		
		int n = 0;
		double sum = 0.0;
		double sum_arith = 0.0;
		double mean = 0.0;
		double meanSqDev = 0.0;
		double sdev = 0.0;
		double maxe = 0.0;
	
		
		for(int i=0; i<sl.size();i++){
			RQSpecies sp = (RQSpecies)sl.elementAt(i);
			if(!sp.getExtinct()){
				sum += log10(sp.getEnergy());
				sum_arith += (sp.getEnergy());
				n += 1;
				if(log10(sp.getEnergy()) > maxe) maxe = log10(sp.getEnergy());
				}
			}
		bigN = n;
		mean = sum/n;
		mean_of_dist = mean;
		double mean_arithmetic = sum_arith/n;
		euler_diff = Math.log(mean_arithmetic) - (Math.log(10.0) * mean); //Difference between ln of mean energy and mean of ln(energy) 
		
		int xi = 0;
		
		for(int i=sl.size()-1;i>-1;i--){
			RQSpecies sp = (RQSpecies)sl.elementAt(i);
			if(!sp.getExtinct()){
				double deviation = ( log10(sp.getEnergy())-mean );
				meanSqDev += Math.pow(deviation,2);
				}
			}
		
		sdev = Math.sqrt(meanSqDev/(n-1));
		
		stdev_of_dist = sdev;
		
		int x = (int)(timestep * 200/timesteps);
		int y = (int)((200/2) * ((-1 * sdev)) + 200);
		
		if(timestep >= 1){
			if (timestep == 1){
				lastx = x;
				lasty = y;
				}
			g.drawLine(lastx,lasty,x,y);
			}
			
		lastx = x;
		lasty = y;
		
		if(timestep==timesteps){
			lastx = 0;
			lasty = 0;
			}
		
		}

	public int getN(){
		return bigN;
		}
		
	public double getEuler(){
		return euler_diff;
		}
		
	public double getArithmeticMean(){
		return mean_arithmetic;
		}
			
	public double getMean(){
		return mean_of_dist;
		}
			
	public double getSdev(){
		return stdev_of_dist;
		}
		
	public void testDraw(){  // Just for testing purposes
		Graphics g = this.getGraphics();
		g.setColor(drawColor);
		
		g.drawLine(40,165,45,160);
		g.drawLine(45,160,50,160);
		g.drawLine(50,160,55,150);
		g.drawLine(55,150,60,142);
		g.drawLine(60,142,65,151);
		g.drawLine(65,151,70,148);
		g.drawLine(70,148,75,133);
		g.drawLine(75,133,80,138);
		g.drawLine(80,138,85,135);
		g.drawLine(85,135,90,144);
		g.drawLine(90,144,95,139);
		g.drawLine(95,139,100,131);
		}
		
	public void repaint(){}

	public void paint(Graphics g) {	
		
		
		
		g.setColor(drawColor);
		
		g.drawLine(40,20,40,220);
		g.drawLine(40,220,270,220);
		g.drawLine(40,120,270,120);
		g.drawLine(40,170,270,170);

		g.drawString("0",40,235);
		g.drawString("Timesteps", 115,240);
		g.drawString("Sigma", 5,90);
		g.drawString("1.0", 5,125);
		g.drawString("2.0", 5,25);
		g.drawString("0.5", 5,175);	
		g.drawString(""+timesteps,240,235);	
	
		
		} 

	public double log10(double x) {
  		return Math.log(x)/Math.log(10.0);  
		}
		

		
}