//
//  SeudsCRT.java
//  Red Queen Simulator
//
//  Created by John Damuth on Thu Jan 27 2005.
//  Copyright (c) 2005 Pantherion Corporation. All rights reserved.
//  Version 1.5

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.io.*;
import java.util.*;
import java.text.DecimalFormat;

public class SeudsCRT extends Canvas {

	Dimension	size;
	Color 		drawColor = new java.awt.Color(175, 210, 40);
	int n = 0;
	double mean_arithmetic = 0.0;
	double mean_of_dist = 0.0;
	double stdev_of_dist = 0.0;
	double euler_diff = 0.0;
	int bigN = 0;


	public SeudsCRT(Dimension sz){
	
		size = sz;
		setBackground(new java.awt.Color(30, 125, 30));
		setSize(size);
		setFont(new java.awt.Font("Helvetica", 0, 10));
		setLocation(new java.awt.Point(5, 8));
		}

	public void draw_graph(int timestep, double slope, double yint, Vector splist, double min_den){	
		Graphics g = this.getGraphics();
		g.setColor(drawColor);
		g.translate(40,20);
		
		g.clearRect(1,-20,220,220);
		
		Vector sl = (Vector)splist.clone();
		
		Comparator energySort = new Comparator() {
			public int compare(Object o1, Object o2) {
			RQSpecies h1 = (RQSpecies) o1;
			RQSpecies h2 = (RQSpecies) o2;
			
			if(h1.getEnergy() < h2.getEnergy())
				return -1;
			else if (h1.getEnergy() > h2.getEnergy())
				return +1;
			else return 0;
			}
		};
		
		Collections.sort(sl, energySort);
		
		n = 0;
		double sum = 0.0;
		double sum_arith = 0.0;
		double mean = 0.0;
		double meanSqDev = 0.0;
		double sdev = 0.0;
		double maxe = 0.0;
	
		
		for(int i=0; i<sl.size();i++){
			RQSpecies sp = (RQSpecies)sl.elementAt(i);
			if(!sp.getExtinct()){
				sum += log10(sp.getEnergy());
				n += 1;
				if(sp.getEnergy() > maxe) maxe = sp.getEnergy();
				}
			}
		bigN = n;
		mean = sum/n;
		mean_of_dist = mean;
		double mean_arithmetic = sum_arith/n;
		euler_diff = Math.log(mean_arithmetic) - (Math.log(10.0) * mean); //Difference between ln of mean energy and mean of ln(energy) 		
		

		
		int xi = 0;
		
		for(int i=sl.size()-1;i>-1;i--){
			RQSpecies sp = (RQSpecies)sl.elementAt(i);
			if(!sp.getExtinct()){
				int x = (int)Math.round(xi * 1.75); xi = xi + 1;
				int y = 10 - (int)Math.round( (log10(sp.getEnergy() / maxe)) * 20);
				double deviation = ( log10(sp.getEnergy())-mean );
				meanSqDev += Math.pow(deviation,2);
				if (y < 200){
					if(sp.IsNew()) g.setColor(Color.red); //To show new species
					else g.setColor(drawColor);
					g.fillOval(x,y,2,2);
					}
				}
			}
		
		sdev = Math.sqrt(meanSqDev/(n-1));
		

		g.setColor(drawColor);
		DecimalFormat df = new DecimalFormat("###.000");
		g.drawString("Std Dev: " + df.format(sdev),40,198);
		g.drawString("S = " + n,180,198);	
		}
		
	public void testDraw(){  // Just for testing purposes
		Graphics g = this.getGraphics();
		g.setColor(drawColor);
		
		g.drawLine(40,165,45,160);
		g.drawLine(45,160,50,160);
		g.drawLine(50,160,55,150);
		g.drawLine(55,150,60,142);
		g.drawLine(60,142,65,151);
		g.drawLine(65,151,70,148);
		g.drawLine(70,148,75,133);
		g.drawLine(75,133,80,138);
		g.drawLine(80,138,85,135);
		g.drawLine(85,135,90,144);
		g.drawLine(90,144,95,139);
		g.drawLine(95,139,100,131);
		}

	public void paint(Graphics g) {	
		
		
		
		g.setColor(drawColor);
		
		g.drawLine(40,20,40,220);
		g.drawLine(40,220,270,220);

		g.drawString("1",40,235);
		g.drawString("Rank Order", 115,240);
		g.drawString("Log 10", 5,110);	
		g.drawString("E-Use", 5,125);
	
		
		} 

	public int getN(){
		return bigN;
		}
		
	public double getEuler(){
		return euler_diff;
		}
		
	public double getArithmeticMean(){
		return mean_arithmetic;
		}
			
	public double getMean(){
		return mean_of_dist;
		}
			
	public double getSdev(){
		return stdev_of_dist;
		}


	public double log10(double x) {
  		return Math.log(x)/Math.log(10.0);  
		}
		

		
}